#include "PCH.h"
#include "Enemy.h"
#include "Level.h"
#include "Field.h"
#include "Explosion.h"
#include "PlayerCannon.h"
#include "gfx.h"

#include "EffectSprite.h"

Enemy::Enemy(Level& level,
			 int type)
	: GameObject(level), type(type)
{
	x = g_hge->Random_Float(64.0f, 13.0f * 64);
	y = g_hge->Random_Float(0.0f, 650.0f);
	vx = vy = 0;
	angle = 0.0f;
	counter = 2.0f;
	age = 0.0f;
	color = hgeColor(g_hge->Random_Float(0.01f, 1.0f),g_hge->Random_Float(0.01f, 1.0f),g_hge->Random_Float(0.01f, 1.0f),1.0f);
	float m = max(max(color.r, color.g), color.b);
	color.r /= m;
	color.g /= m;
	color.b /= m;

	findTarget();

	hgeParticleSystemInfo *psi = (hgeParticleSystemInfo*)g_hge->Resource_Load("../data/fire.psi");
	psi->sprite = g_sprFire;
	psi->nEmission = 25;
	psi->fSizeStart = 0.25f;
	psi->fSizeEnd = 0.0f;
	psi->colColorStart=hgeColor(1.0f,1.0f,0.0f,0.25f);
	psi->colColorEnd=hgeColor(0.0f,0.0f,1.0f,0.0f);
	particleSys = level.getParticleMgr()->SpawnPS(psi, 0.0f, 0.0f);
}

Enemy::~Enemy()
{
	level.getParticleMgr()->KillPS(particleSys);
}

void Enemy::update()
{
	x += vx;
	y += vy;

	Field* target = level.getField(targetX, targetY);
	if(!target || target->type == 7) findTarget();

	float tx = 32.0f + 64 + 64 * targetX;
	float ty = 16.0f + 32 * targetY;

	float dx = tx - x;
	float dy = ty - y;
	float d = sqrtf(dx*dx + dy*dy);
	dx /= d; dy /= d;
	float targetAngle = atan2f(dy, dx) + M_PI / 2;
	angle = (0.95f * angle) + 0.05f * targetAngle;

	if(d < 16.0f)
	{
		vx *= 0.75f;
		vy *= 0.75f;
		counter -= 0.01f;

		if(counter <= 0.0f)
		{
			if(g_hge->Random_Int(0, 3) == 0)
			{
				// Block blockieren
				Field* field = level.getField(targetX, targetY);
				if(field)
				{
					field->type = 7;
					counter = 2.0f;

					for( int i = 0; i < 10; ++i )
					{
						EffectSprite* pNew= new EffectSprite( level );
						pNew->x= 96.0f + g_fBlockWidth * targetX;// + g_hge->Random_Float( -10.0f, 10.0f );
						pNew->y= g_fBlockHeight * targetY;
						pNew->rot=0.0f;

						pNew->dx= g_hge->Random_Float( -0.9f, 0.9f );
						pNew->dy= -0.5f + g_hge->Random_Float( -0.3f, 0.3f );
						pNew->drot= pNew->dx * 0.1f;
						pNew->g= 0.01f;
						pNew->age= 0.0f;
						pNew->maxage= 3.0f;

						pNew->scale= g_hge->Random_Float( 0.4f, 0.7f );

						pNew->sprSprite= g_sprRockEffect;
						//pNew->color.SetHWColor(color);

						level.adoptObject( pNew );
					}
				}
			}
			else
			{
				findTarget();
				counter = 2.0f;
			}
		}
	}
	else
	{
		counter = 2.0f;
	}

	float a = 0.0085f;
	vx += dx * a * g_hge->Random_Float(0.5f, 1.5f);
	vy += dy * a * g_hge->Random_Float(0.5f, 1.5f);
	angle += g_hge->Random_Float(-0.025f, 0.025f);

	vx *= 0.99f;
	vy *= 0.99f;

	if(x < 0.0f) x = 0.0f, vx *= -0.9f;
	else if(x > 14.0f * 64) x = 14.0f * 64, vx *= -0.9f;
	if(y < 0.0f) y = 0.0f, vy *= -0.9f;
	else if(y > 650.0f) y = 650.0f, vy *= -0.9f;

	age += 0.01f;
	if(age < 1.0f)
	{
		angle = age * 4 * M_PI;
	}

	particleSys->MoveTo(x, y);
}

void Enemy::render()
{
	float scale = 1.0f;
	if(age < 1.0f) scale = age;
	g_sprEnemy[type]->SetColor(color.GetHWColor());
	g_sprEnemy[type]->RenderEx(x, y, angle, scale);
	
	if(type == 1)
	{
		float up_x = sinf(angle) * scale;
		float up_y = -cosf(angle) * scale;
		float right_x = -up_y;
		float right_y = up_x;
		PlayerCannon* cannon = level.getPlayerCannon();
		float aimX = cannon->getAimX();
		float aimY = cannon->getAimY();

		float eyeX[] = { -10.0f, 10.0f };
		float eyeY[] = { -5.0f, -5.0f };
		for(int i = 0; i < 2; i++)
		{
			float worldX = x + eyeX[i] * right_x + eyeY[i] * up_x;
			float worldY = y + eyeX[i] * right_y + eyeY[i] * up_y;
			float dx = aimX - worldX;
			float dy = aimY - worldY;
			float d = sqrtf(dx * dx + dy * dy);
			dx /= d;
			dy /= d;
			float angleToCrosshair = atan2f(dy, dx) - M_PI / 2;
			g_sprEnemy[7]->RenderEx(worldX,
									worldY,
									angleToCrosshair + g_hge->Random_Float(-0.1f, 0.1f),
									scale);
		}

	}
}

void Enemy::onHitByExplosion()
{
	Explosion* explosion = new Explosion(level, x, y);
	level.adoptObject(explosion);
	level.addPoints(10, x, y);
	level.m_EnemiesKilled++; 
	
	level.m_iCombos++;
	level.m_iResetCombo= 3 * Level::c_iComboTime;

	dead = true;
}

void Enemy::findTarget()
{
	targetX = g_hge->Random_Int(0, LEVEL_WIDTH - 1);
	targetY = g_hge->Random_Int(0, LEVEL_HEIGHT - 1);
}