#include "PCH.h"
#include "Level.h"
#include "Field.h"
#include "GameObject.h"
#include "Enemy.h"
#include "Player.h"
#include "PlayerCannon.h"
#include "Rocket.h"
#include "Enemy.h"
#include "gfx.h"
#include "TargetValue.h"
#include "EffectSprite.h"

#include <algorithm>
#include <string>

Level::Level()
	: m_DownValue( 0.0f, 0.0f, 0.0f, 0.02f, 0.85f ),
		m_PointsX( 0.0f, 0.0f, 0.0f, 0.02f, 0.85f ),
		m_PointsY( 0.0f, 0.0f, 0.0f, 0.02f, 0.85f ),
		m_GameOverX( 0.0f, 0.0f, 0.0f, 0.005f, 0.95f ),
	  m_fOffsetX(0.0f), m_fOffsetY(0.0f), 
	  m_iTimeToLine(0),
	  numRockets(0), numEnemies(0)
{
	m_iTimeToCombo= 0;
	m_iResetCombo= 0;
	m_iCombos= 0;
	m_iStones= 0;
	m_iTimeToEnemySpawn = 0;

	m_bFUCKHGE= true;
	m_bGameOver= false;

	particleMgr = new hgeParticleManager();

	for(int x = 0; x < LEVEL_WIDTH; x++)
	{
		for(int y = 0; y < LEVEL_HEIGHT; y++)
		{
			if(y < 10 )
			{
				fields[x][y]= createNewField( FT_NORMAL );
			}
			else fields[x][y]=0;
		}
	}

	m_iStage= 0;
	m_iSubStage= 0;
	m_iPoints= 0;

	m_EnemiesKilled= 0;
	m_StonesKilled= 0;

	m_fLevelUp= 0.0f;

	for( int i= 0; i< 3; i++ )
	{
		CPlayerShip* pShip= new CPlayerShip( *this );

		pShip->m_iColor= i;

		switch( i )
		{
		case 0:
			pShip->m_iKeyUp= HGEK_UP;
			pShip->m_iKeyDown= HGEK_DOWN;
			pShip->m_iKeyLeft= HGEK_LEFT;
			pShip->m_iKeyRight= HGEK_RIGHT;
			break;
		case 1:
			pShip->m_iKeyUp= HGEK_W;
			pShip->m_iKeyDown= HGEK_S;
			pShip->m_iKeyLeft= HGEK_A;
			pShip->m_iKeyRight= HGEK_D;
			break;
		case 2:
			pShip->m_iKeyUp= HGEK_I;
			pShip->m_iKeyDown= HGEK_K;
			pShip->m_iKeyLeft= HGEK_J;
			pShip->m_iKeyRight= HGEK_L;
			break;
		}

		pShip->m_fX= 64.0f;
		pShip->m_fY= 640.0f;

		pShip->m_fDepth= 16.0f - i * 8.0f;

		pShip->m_iRow= 5 + i;

		m_playerShips[ i ]= pShip;
		objects.push_back( pShip );
	}

	m_playerCannon = new PlayerCannon(*this);
	objects.push_back(m_playerCannon);

	m_fGirlTime= 0.0f;
	m_iGirlNumber= -1;

	m_fGirlTime= 10.0f;
	m_iGirlNumber= 0;
	m_GirlText= "Use W,A,S,D / I,J,K,L / cursor keys and the mouse!";
	m_fGirlIntensity= 0.0f;
	//StartGirl( 0.1f );
}

Level::~Level()
{
	for(int x = 0; x < LEVEL_WIDTH; x++)
	{
		for(int y = 0; y < LEVEL_HEIGHT; y++)
		{
			delete fields[x][y];
		}
	}

	for(std::vector<GameObject*>::iterator it = objects.begin();
		it != objects.end();
		it++)
	{
		deleteObject(*it);
	}

	delete particleMgr;
}

void Level::render() 
{
	g_sprBackground->Render(0, 0);

	if( m_iGirlNumber >= 0 && m_fGirlTime >=0 )
	{
		float fPos= 0.0f;
		if( m_fGirlTime <  1.0f )
		{
			fPos= 1.0f - m_fGirlTime;
		}
		if( m_fGirlTime > 9.0f )
		{
			fPos= m_fGirlTime - 9.0f;
		}

		fPos= fPos * fPos;
		fPos= 832.0f + fPos * 192.0f;

		g_sprGirls[ m_iGirlNumber ]->RenderEx( fPos, 300.0f, 0.0f, 0.7f );
		g_sprSpeakBubble->RenderEx( fPos - 20.0f, 500.0f, 0.0f, 1.0f );

		g_fntSmall->SetColor( 0xff000000 );
		g_fntSmall->SetScale( 1.0f );
		g_fntSmall->printfb(	
						fPos + 5.0f,
						555.0f,
						170, 70,
						HGETEXT_CENTER | HGETEXT_MIDDLE,
						m_GirlText.c_str() );

	}

	for(int x = 0; x < LEVEL_WIDTH; x++)
	{
		for(int y = LEVEL_HEIGHT - 1 ; y >= 0; y--)
		{
			float fX= g_fBlockWidth * (float)x + m_fOffsetX;
			float fY= g_fBlockHeight * (float)y + m_fOffsetY;

			if(fields[x][y]) fields[x][y]->render( fX, fY);
		}
	}

	if( objects.size() > 0 )
	{
		std::sort( objects.begin(), objects.end(), **objects.begin() );
	}

	for(std::vector<GameObject*>::iterator it = objects.begin();
		it != objects.end();
		it++)
	{
		(*it)->render();
	}

	particleMgr->Render();

	if( m_iCombos > 0 )
	{
		g_fntBig->SetScale( 1.0f );
		g_fntBig->SetColor( 0x80000000 );
		g_fntBig->printf(	516.0f + m_PointsX.m_fValue,
							104.0f + m_PointsY.m_fValue,
							HGETEXT_CENTER,
							"Stones: %d x%d\n= %d", m_iStones, m_iCombos, m_iCombos * m_iCombos * m_iStones );

		g_fntBig->SetColor( 0xffff8000 );
		g_fntBig->printf(	512.0f + m_PointsX.m_fValue,
							100.0f + m_PointsY.m_fValue,
							HGETEXT_CENTER,
							"Stones: %d x%d\n= %d", m_iStones, m_iCombos, m_iCombos * m_iCombos * m_iStones );
	}

	g_fntSmall->SetColor( 0x80000000 );
	g_fntSmall->SetScale( 1.0f );
	g_fntSmall->printf(	900.0f,
						45.0f,
						HGETEXT_LEFT,
						"Level %d\n\nScore:\n%08d\n\nStones:\n%08d\n\nEnemies:\n%08d",
						m_iStage+1,m_iPoints,m_StonesKilled,m_EnemiesKilled);
	g_fntSmall->SetColor( 0xffffffff );
	g_fntSmall->printf(	902.0f,
		47.0f,
		HGETEXT_LEFT,
		"Level %d\n\nScore:\n%08d\n\nStones:\n%08d\n\nEnemies:\n%08d",
		m_iStage+1,m_iPoints,m_StonesKilled,m_EnemiesKilled);

	if( m_fLevelUp > 0.0f )
	{
		float alpha = 1.0f - m_fLevelUp / 1.0f;
		if(alpha > 1.0f) alpha = 1.0f;
		else if(alpha < 0.0f) alpha = 0.0f;
		hgeColor color(1.0f, 0.5f, 0.0f, 1.0f - alpha);

		g_fntBig->SetScale( 2.0f * alpha + 2.0f );
		g_fntBig->SetColor( hgeColor(0.0f, 0.0f, 0.0f, 0.5f * color.a).GetHWColor());
		g_fntBig->printf(	516.0f,
			388.0f,
			HGETEXT_CENTER,
			"Level %d", m_iStage + 1);
		g_fntBig->SetColor( color.GetHWColor() );
		g_fntBig->printf(	512.0f,
							384.0f,
							HGETEXT_CENTER,
							"Level %d", m_iStage + 1);
	}

	if( m_bGameOver )
	{
		g_sprGameOver->Render( 512.0f + m_GameOverX.m_fValue, 384.0f );
	}
}

void Level::update()
{
	int TimeStep= c_iTimeStep;
	
	m_iTimeToLine-= TimeStep;

	if( m_iTimeToLine <= 0 )
	{
		if( newLine() > 0 )
		{
			if( !m_bGameOver )
			{
				m_GameOverX.m_fValue= -800.0f;
			}
			m_bGameOver= true;
		}
	}

	m_iTimeToEnemySpawn -= TimeStep;

	if( m_iTimeToEnemySpawn <= 0 )
	{
		newEnemy();
	}

	m_PointsX.timeStep();
	m_PointsY.timeStep();

	m_GameOverX.timeStep();

	m_DownValue.timeStep();
	m_fOffsetY= m_DownValue.m_fValue - 32.0f;
	m_fOffsetX= 64.0f;

	for(std::vector<GameObject*>::iterator it = objects.begin();
		it != objects.end();)
	{
		GameObject* obj = *it;
		obj->update();
		if(obj->isDead())
		{
			deleteObject(obj);
			it = objects.erase(it);
		}
		else it++;
	}

	// neue Objekte reintun
	objects.insert(objects.end(), objectsToAdd.begin(), objectsToAdd.end());
	objectsToAdd.clear();

	for(int x = 0; x < LEVEL_WIDTH; x++)
	{
		for(int y = LEVEL_HEIGHT - 1 ; y >= 0; y--)
		{
			float fX= g_fBlockWidth * (float)x + m_fOffsetX;
			float fY= g_fBlockHeight * (float)y + m_fOffsetY;

			if(fields[x][y]) fields[x][y]->update();
		}
	}

	particleMgr->Update(0.01f);

	checkCombo();

	m_fLevelUp-= 0.01f;

	if( m_iGirlNumber >= 0 && m_fGirlTime >= 0.0f )
	{
		m_fGirlTime-= 0.01f;

		if( m_fGirlTime > 1.0f && m_fGirlTime < 9.0f && g_hge->Random_Float( 0.0f, 1.0f ) < m_fGirlIntensity )
		{
			EffectSprite* pNew= new EffectSprite( *this );
			pNew->x= 832.0f + 96.0f;// + g_hge->Random_Float( -10.0f, 10.0f );
			pNew->y= 300.0f;
			pNew->rot=0.0f;

			pNew->dx= g_hge->Random_Float( -0.6f, 0.6f );
			pNew->dy= -0.2f + g_hge->Random_Float( -0.1f, 0.1f );
			pNew->drot= -pNew->dx * 0.01f;
			pNew->g= -0.01f;
			pNew->age= 0.0f;
			pNew->maxage= 2.0f;

			pNew->scale= 0.4f;

			pNew->sprSprite= g_sprHeartEffect;

			adoptObject( pNew );
		}
	}
}

int Level::newLine()
{
	int iRet=0;

	for(int x = 0; x < LEVEL_WIDTH; x++)
	{
		if( fields[x][ LEVEL_HEIGHT - 1] != 0 )
		{
			iRet++;
			delete fields[x][ LEVEL_HEIGHT - 1];
		}
	}

	for( int i= 0; i < (int)m_ActiveFields.size(); ++i )
	{
		m_ActiveFields[ i ].m_iY++;
	}

	for(int x = 0; x < LEVEL_WIDTH; x++)
	{
		for(int y = LEVEL_HEIGHT - 2 ; y >= 0; y--)
		{
			fields[x][y + 1] = fields[x][y];
			if( fields[x][y + 1] != 0 )
			{
				//fields[x][y + 1]->m_tvY.m_fValue= g_hge->Random_Float( -50.0f, 50.0f );
				//fields[x][y + 1]->m_tvX.m_fValue= g_hge->Random_Float( -50.0f, 50.0f );
			}
		}
	}

	for(int x = 0; x < LEVEL_WIDTH; x++)
	{
		fields[x][0] =createNewField( FT_NORMAL );
	}
	
	{
		float fWeight= (float)min( m_iStage, c_iFinalStage) / (float)c_iFinalStage;
		m_iTimeToLine= (int)( fWeight * (float)c_iTimeToNextLineB+
								( 1.0f - fWeight ) * (float)c_iTimeToNextLineA );
	}
	m_DownValue.m_fValue= -32.0f;

	return iRet;
}

void Level::newEnemy()
{
	if(numEnemies >= 8) return;

	adoptObject(new Enemy(*this, g_hge->Random_Int(0, 1)));

	float fWeight= (float)min( m_iStage, c_iFinalStage) / (float)c_iFinalStage;
	m_iTimeToEnemySpawn= (int)( fWeight * (float)c_iTimeToEnemySpawnB+
		( 1.0f - fWeight ) * (float)c_iTimeToEnemySpawnA );
}

Field* Level::createNewField( FieldType Type )
{
	switch( Type )
	{
	case FT_NORMAL:
		return new Field( rand() % 6 );
	case FT_BLOCKED:
		return new Field(7);
	}

	return 0;
}

void Level::adoptObject(GameObject* p_obj)
{
	objectsToAdd.push_back(p_obj);
	if(dynamic_cast<Rocket*>(p_obj)) numRockets++;
	else if(dynamic_cast<Enemy*>(p_obj)) numEnemies++;
}

void Level::deleteObject(GameObject* p_obj)
{
	if(dynamic_cast<Rocket*>(p_obj)) numRockets--;
	else if(dynamic_cast<Enemy*>(p_obj)) numEnemies--;
	delete p_obj;
}

std::vector<Enemy*> Level::getEnemiesNear(float x, float y, float radius)
{
	std::vector<Enemy*> enemies;
	for(std::vector<GameObject*>::iterator it = objects.begin();
		it != objects.end();
		it++)
	{
		Enemy* enemy = dynamic_cast<Enemy*>(*it);
		if(enemy)
		{
			float dx = x - enemy->getX();
			float dy = y - enemy->getY();
			float d = dx*dx + dy*dy;
			if(d <= radius * radius)
			{
				enemies.push_back(enemy);
			}
		}
	}

	return enemies;
}

void Level::addAmmo(int num)
{
	m_playerCannon->addAmmo(num);
}