#ifndef LEVEL_H
#define LEVEL_H

#define LEVEL_WIDTH 12
#define LEVEL_HEIGHT 24

class Field;
class GameObject;
#include "TargetValue.h"

#include <set>
#include <string.h>

class CPlayerShip;
class PlayerCannon;
class Enemy;

class Level
{
public:
	Level();
	~Level();

	void render();
	void update();

	int newLine();
	void newEnemy();

	hgeParticleManager* getParticleMgr() { return particleMgr; }

	Field* GetLowestFromRow( int iRow, float& fLastPosY );
	float GetLowestPos( int iRow );
	void movePlayerDown( int iColor );
	int LowestPlayer( int iRow );

	void putBack( int iRow, Field* pField, float fPosY );

	void adoptObject(GameObject* p_obj);
	int getNumRockets() { return numRockets; }

	std::vector<Enemy*> getEnemiesNear(float x, float y, float radius);
	Field* getField(int x, int y) { return fields[x][y]; }

	PlayerCannon* getPlayerCannon() { return m_playerCannon; }

	bool m_bFUCKHGE;
	bool m_bGameOver;

	void addPoints( int iPoints, float x = -1.0f, float y = -1.0f );
	void addLevel( int iSubLevel );

	void StartGirl(  float fInten  );

	int m_iTimeToCombo;
	int m_iResetCombo;
	int	m_iCombos;
	int	m_iStones;
	float cx, cy;

	int m_EnemiesKilled;
	int m_StonesKilled;

	// Balancing...
	static const int c_iTimeStep= 10000;
	static const int c_iTimeToNextLineA= 10000000;
	static const int c_iTimeToNextLineB= 1000000;
	static const int c_iFinalStage= 100;
	static const int c_iComboTime= 10000 * 100;
	static const int c_iTimeToEnemySpawnA = 15000000;
	static const int c_iTimeToEnemySpawnB = 500000;

	float	m_fGirlTime;
	int		m_iGirlNumber;
	std::string m_GirlText;
	float	m_fGirlIntensity;

	//Ende
private:

	Field* fields[LEVEL_WIDTH][LEVEL_HEIGHT];
	
	enum FieldType
	{
		FT_NORMAL,
		FT_BLOCKED
	};

	Field* createNewField( FieldType Type );

	void deleteObject(GameObject* p_obj);

	float m_fOffsetX;
	float m_fOffsetY;

	int m_iTimeToLine;
	STargetValue m_DownValue;
	STargetValue m_PointsX;
	STargetValue m_PointsY;

	STargetValue m_GameOverX;

	int m_iTimeToEnemySpawn;

	int m_iStage;
	int m_iSubStage;
	int m_iPoints;

	float m_fLevelUp;

	// Objekte
	std::vector<GameObject*> objects;

	// neue Objekte
	std::vector<GameObject*> objectsToAdd;

	hgeParticleManager* particleMgr;
	
	static const int c_iShipCount= 3;
	CPlayerShip* m_playerShips[ c_iShipCount ];
	PlayerCannon* m_playerCannon;

	struct SHint
	{
		int m_iX;
		int m_iY;

		bool operator<( const SHint& h )const
		{
			if( m_iX == h.m_iX )
			{
				return m_iY < h.m_iY;
			}
			return m_iX < h.m_iX;
		}
	};
	std::vector<SHint> m_ActiveFields;

	void checkCombo();
	void checkNeighbours( std::set< SHint >& hints, int iType );
	void crush();

	int numRockets;
	int numEnemies;

	void addAmmo( int );
};

#endif